@@ -5,8 +5,8 @@
 * Botan is released under the Simplified BSD License (see license.txt)
 */
 
-#ifndef BOTAN_KECCAK_H__
-#define BOTAN_KECCAK_H__
+#ifndef BOTAN_KECCAK_H_
+#define BOTAN_KECCAK_H_
 
 #include <botan/hash.h>
 #include <botan/secmem.h>
@@ -17,7 +17,7 @@ namespace Botan {
 /**
 * Keccak[1600], a SHA-3 candidate
 */
-class BOTAN_DLL Keccak_1600 final : public HashFunction
+class BOTAN_PUBLIC_API(2,0) Keccak_1600 final : public HashFunction
    {
    public:
 
@@ -31,6 +31,7 @@ class BOTAN_DLL Keccak_1600 final : public HashFunction
       size_t output_length() const override { return m_output_bits / 8; }
 
       HashFunction* clone() const override;
+      std::unique_ptr<HashFunction> copy_state() const override;
       std::string name() const override;
       void clear() override;
 
